#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct date_s {
    int year;
    int month;
    int day;
};

enum date_value_e {
    YEAR,
    MONTH,
    DAY,
};

struct date_s date_from_string(const char* s)
{
    struct date_s d;
#if 1
    sscanf(s, "%d/%d/%d", &d.day, &d.month, &d.year);
#else
    char buffer[5];

    memcpy(buffer, s, 2*sizeof(char));
    buffer[2] = '\0';
    d.day = atoi(buffer);

    memcpy(buffer, s+3, 2*sizeof(char));
    // buffer[2] = '\0'; // inutile
    d.month = atoi(buffer);

    d.year = atoi(s+6);
#endif

    return d;
}

void date_to_string(const struct date_s* const date, char sep, char* str)
{
    sprintf(str, "%02d%c%02d%c%d", date->day, sep, date->month, sep, date->year);
}

int date_compare(const struct date_s* const d1, const struct date_s* const d2)
{
#if 0
    if (d1->year < d2->year)
    {
        return -1;
    }
    else if (d1->year > d2->year)
    {
        return 1;
    }
    else
    {
        if (d1->month < d2->month)
        {
            return -1;
        }
        else if (d1->month > d2->month)
        {
            return 1;
        }
        else
        {
            return d1->day - d2->day;
        }
    }
#else
    return (d1->year * 365 + d1->month * 31 + d1->day) - (d2->year * 365 + d2->month * 31 + d2->day);
#endif
}

int date_get_value(const struct date_s* const date, const enum date_value_e value)
{
    switch (value)
    {
        case YEAR:
            return date->year;
        case MONTH:
            return date->month;
        case DAY:
            return date->day;
        default:
            return -1;
    }
}

char* month_to_str(const int month)
{
    if (month < 1 || month > 12)
    {
        return "Error";
    }

    char* months[] = {
        "January",
        "February",
        "March",
        "April",
        "May",
        "June",
        "July",
        "August",
        "September",
        "October",
        "November",
        "December",
    };

    return months[month-1];
}

int date_is_bissextile(const struct date_s* const date)
{
    const int year = date->year;
    return (year % 4 == 0 && year % 100 != 0) || (year % 400 == 0);
}


int main(int argc, char* argv[])
{
    struct date_s d = date_from_string("01/11/2023");
    struct date_s d_before = date_from_string("31/10/2023");
    struct date_s d_after = date_from_string("02/11/2023");

    char buffer[11];
    date_to_string(&d, '-', buffer);
    printf("%s\n", buffer);

    assert(date_compare(&d, &d) == 0);
    assert(date_compare(&d_before, &d) < 0);
    assert(date_compare(&d, &d_before) > 0);
    assert(date_compare(&d_after, &d) > 0);
    assert(date_compare(&d, &d_after) < 0);

    assert(date_get_value(&d, YEAR) == 2023);
    assert(date_get_value(&d, MONTH) == 11);
    assert(date_get_value(&d, DAY) == 1);
    assert(date_get_value(&d, 42) == -1);

    assert(strcmp(month_to_str(10), "October") == 0);

    assert(date_is_bissextile(&d) == 0);

    return EXIT_SUCCESS;
}
